﻿// --------------------------------------------------------------------------------------
//
// DrehMain.cs                                                    Ulrich Müller  14.07.07
//                                                                PCS1700 C# 2005 Express
// Drehzahlmessung eines fischertechnik Motors
// Motor an M2, Lichtschranke an M1 / I1
//                Basis ist die Assembly FishFace2005.DLL
//
// --------------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using FishFace40;

namespace Drehzahl {
  public partial class Drehzahl : Form {
    const double WalzenradDurchmesser = 52;
    const double SchneckeZahn         = 20;
    const double ImpulseUmdrehung     = 12;
    const Out Lampe                   = Out.M1;
    const int Photo                   = (int)Inp.I1;
    const Out Motor                   = Out.M2;

    double MotorradDurchmesser;
    double MessDauer;
    int Zeit;
    FishFace ft = new FishFace();

    private void Action() {
      ft.SetMotor(Lampe, Dir.On);
      ft.SetMotor(Motor, Dir.Left);
      ft.Pause(1234);
      ft.ClearCounter(Photo);
      Zeit                = 0;
      MotorradDurchmesser = (int)numMotorRolle.Value;
      MessDauer           = (int)numMessDauer.Value;
      prgMessung.Maximum  = (int)MessDauer;
      prgMessung.Value    = 0;
      prgMessung.Visible  = true;
      tmrAction.Enabled   = true;

      ft.Pause((int)MessDauer * 1000);

      ft.SetMotor(Lampe, Dir.Off);
      ft.SetMotor(Motor, Dir.Off);
      tmrAction.Enabled  = false;
      prgMessung.Visible = false;

      lblErgebnis.Text = DrehzahlMotor(MessDauer, 
        ft.GetCounter(Photo)).ToString("Drehzahl : 0  1/min");
    }

    #region --- Programm-Kontrolle ---
    public Drehzahl() {
      InitializeComponent();
    }

    private void cmdAction_Click(object sender, EventArgs e) {
      // --- Open erstes ROBO an USB oder Intelligent Interface an COM
      //     während des Ablaufs von Action() gesperrt ---------------
      try {
        if (cboPortName.SelectedIndex == 0)
          ft.OpenInterface(IFTypen.ftROBO_first_USB, 0);
        else ft.OpenInterface(IFTypen.ftIntelligent_IF, cboPortName.SelectedIndex, 5);
        cmdAction.Enabled = false;
        cmdEnde.Text = "&HALT";
        lblStatus.Text = "--- Bei der Arbeit ---";
        lblHinweis.Text = "--- Ende : HALT-Button oder ESC-Taste ---";

        Action();        // --- Haupt-Routine der Anwendung

        ft.ClearMotors();
      }
      catch (FishFaceException eft) {
        MessageBox.Show(eft.Message, this.Text,
          MessageBoxButtons.OK, MessageBoxIcon.Stop);
      }
      finally {
        ft.CloseInterface();
        cmdAction.Enabled = true;
        cmdEnde.Text = "&ENDE";
        lblStatus.Text = "--- Auf ein Neues ---";
        lblHinweis.Text = "Nochmal : ACTION, sonst ENDE oder ESC-Taste";
        cmdAction.Focus();
      }
    }

    private void cmdEnde_Click(object sender, EventArgs e) {
      // --- Programmende nur, wenn Action nicht läuft ---
      if (cmdEnde.Text == "&HALT") ft.NotHalt = true; else this.Close();
    }

    private void formLoad(object sender, EventArgs e) {
      // --- Default-Interface : Erstes ROBO Interface an USB ----
      cboPortName.SelectedIndex = 0;
    }

    private void formClosing(object sender, FormClosingEventArgs e) {
      // --- Wenn Action läuft, wird ein Programmabbruch über (x) unterbunden ------
      if (cmdEnde.Text == "&HALT") e.Cancel = true;
    }
    #endregion

    private void tmrAction_Tick(object sender, EventArgs e) {
      prgMessung.Value = ++Zeit;
    }
    private int DrehzahlMotor(double Dauer, double Impulse) {
      double DrehzahlRolle = Impulse / ImpulseUmdrehung * (60.0 / Dauer);
      double Uebersetzung = WalzenradDurchmesser * SchneckeZahn / MotorradDurchmesser;
      return (int)(DrehzahlRolle * Uebersetzung);
    }
  }
}